@@ -601,7 +601,11 @@ int igraph_arpack_rssort(igraph_vector_t *values, igraph_matrix_t *vectors,
 
   IGRAPH_CHECK(igraph_vector_init_seq(&order, 0, nconv-1));
   IGRAPH_FINALLY(igraph_vector_destroy, &order);
+#ifdef HAVE_GFORTRAN
+  igraphdsortr_(sort, &apply, &nconv, d, VECTOR(order), /*which_len=*/ 2);
+#else
   igraphdsortr_(sort, &apply, &nconv, d, VECTOR(order));
+#endif
 
   /* BE is special */
   if (which('B','E')) {
@@ -684,7 +688,11 @@ int igraph_arpack_rnsort(igraph_matrix_t *values, igraph_matrix_t *vectors,
   
   IGRAPH_CHECK(igraph_vector_init_seq(&order, 0, nconv-1));
   IGRAPH_FINALLY(igraph_vector_destroy, &order);
+#ifdef HAVE_GFORTRAN
+  igraphdsortc_(sort, &apply, &nconv, dr, di, VECTOR(order), /*which_len=*/ 2);
+#else
   igraphdsortc_(sort, &apply, &nconv, dr, di, VECTOR(order));
+#endif
 
   if (values) {
     IGRAPH_CHECK(igraph_matrix_resize(values, nans, 2));
@@ -730,25 +738,29 @@ int igraph_arpack_rnsort(igraph_matrix_t *values, igraph_matrix_t *vectors,
 void igraph_i_arpack_auto_ncv(igraph_arpack_options_t* options) {
   /* This is similar to how Octave determines the value of ncv, with some
    * modifications. */
-	int min_ncv = options->nev * 2 + 1;
+  int min_ncv = options->nev * 2 + 1;
 
-	/* Use twice the number of desired eigenvectors plus one by default */
+  /* Use twice the number of desired eigenvectors plus one by default */
   options->ncv = min_ncv;
-	/* ...but use at least 20 Lanczos vectors... */
+  /* ...but use at least 20 Lanczos vectors... */
   if (options->ncv < 20) {
     options->ncv = 20;
   }
-	/* ...but having ncv close to n leads to some problems with small graphs
-	 * (example: PageRank of "A <--> C, D <--> E, B"), so we don't let it
-	 * to be larger than n / 2...
-	 */
+  /* ...but having ncv close to n leads to some problems with small graphs
+   * (example: PageRank of "A <--> C, D <--> E, B"), so we don't let it
+   * to be larger than n / 2...
+   */
   if (options->ncv > options->n / 2) {
     options->ncv = options->n / 2;
   }
-	/* ...but we need at least min_ncv. */
-	if (options->ncv < min_ncv) {
-		options->ncv = min_ncv;
-	}
+  /* ...but we need at least min_ncv. */
+  if (options->ncv < min_ncv) {
+    options->ncv = min_ncv;
+  }
+  /* ...but at most n-1 */
+  if (options->ncv > options->n) {
+    options->ncv = options->n;
+  }
 }
 
 /**
@@ -884,11 +896,20 @@ int igraph_arpack_rssolve(igraph_arpack_function_t *fun, void *extra,
 
   /* Ok, we have everything */
   while (1) {
+#ifdef HAVE_GFORTRAN
+    igraphdsaupd_(&ido, options->bmat, &options->n, options->which,
+		  &options->nev, &options->tol,
+		  resid, &options->ncv, v, &options->ldv,
+		  options->iparam, options->ipntr,
+		  workd, workl, &options->lworkl, &options->info,
+		  /*bmat_len=*/ 1, /*which_len=*/ 2);
+#else
     igraphdsaupd_(&ido, options->bmat, &options->n, options->which,
     		  &options->nev, &options->tol,
     		  resid, &options->ncv, v, &options->ldv,
     		  options->iparam, options->ipntr,
     		  workd, workl, &options->lworkl, &options->info);
+#endif
 
     if (ido==-1 || ido==1) {
       igraph_real_t *from=workd+options->ipntr[0]-1;
@@ -908,12 +929,23 @@ int igraph_arpack_rssolve(igraph_arpack_function_t *fun, void *extra,
   }
   
   options->ierr=0;
+#ifdef HAVE_GFORTRAN
+  igraphdseupd_(&rvec, all, select, d, v, &options->ldv,
+		&options->sigma, options->bmat, &options->n,
+		options->which, &options->nev, &options->tol,
+		resid, &options->ncv, v, &options->ldv, options->iparam,
+		options->ipntr, workd, workl, &options->lworkl,
+		&options->ierr, /*howmny_len=*/ 1, /*bmat_len=*/ 1,
+		/*which_len=*/ 2);
+#else
   igraphdseupd_(&rvec, all, select, d, v, &options->ldv,
   		&options->sigma, options->bmat, &options->n,
   		options->which, &options->nev, &options->tol,
   		resid, &options->ncv, v, &options->ldv, options->iparam,
   		options->ipntr, workd, workl, &options->lworkl,
   		&options->ierr);
+#endif
+
   
   if (options->ierr != 0) {
     IGRAPH_ERROR("ARPACK error", igraph_i_arpack_err_dseupd(options->ierr));
@@ -1091,11 +1123,21 @@ int igraph_arpack_rnsolve(igraph_arpack_function_t *fun, void *extra,
   
   /* Ok, we have everything */
   while (1) {
+#ifdef HAVE_GFORTRAN
+    igraphdnaupd_(&ido, options->bmat, &options->n, options->which,
+		  &options->nev, &options->tol,
+		  resid, &options->ncv, v, &options->ldv,
+		  options->iparam, options->ipntr,
+		  workd, workl, &options->lworkl, &options->info,
+		  /*bmat_len=*/ 1, /*which_len=*/ 2);
+
+#else
     igraphdnaupd_(&ido, options->bmat, &options->n, options->which,
     		  &options->nev, &options->tol,
     		  resid, &options->ncv, v, &options->ldv,
     		  options->iparam, options->ipntr,
     		  workd, workl, &options->lworkl, &options->info);
+#endif
     
     if (ido==-1 || ido==1) {
       igraph_real_t *from=workd+options->ipntr[0]-1;
@@ -1115,12 +1157,22 @@ int igraph_arpack_rnsolve(igraph_arpack_function_t *fun, void *extra,
   }
 
   options->ierr=0;
+#ifdef HAVE_GFORTRAN
+  igraphdneupd_(&rvec, all, select, dr, di, v, &options->ldv,
+		&options->sigma, &options->sigmai, workev, options->bmat,
+		&options->n, options->which, &options->nev, &options->tol,
+		resid, &options->ncv, v, &options->ldv, options->iparam,
+		options->ipntr, workd, workl, &options->lworkl,
+		&options->ierr, /*howmny_len=*/ 1, /*bmat_len=*/ 1,
+		/*which_len=*/ 2);
+#else
   igraphdneupd_(&rvec, all, select, dr, di, v, &options->ldv,
   		&options->sigma, &options->sigmai, workev, options->bmat,
   		&options->n, options->which, &options->nev, &options->tol,
   		resid, &options->ncv, v, &options->ldv, options->iparam,
   		options->ipntr, workd, workl, &options->lworkl,
   		&options->ierr);
+#endif
 
   if (options->ierr != 0) {
     IGRAPH_ERROR("ARPACK error", igraph_i_arpack_err_dneupd(options->info));
